package no.tornado.databinding.converter;

import java.util.HashMap;
import java.util.Map;
import java.util.ServiceLoader;

public class ConverterRegistry {
    private static Map<Class, Map<Class, Converter>> forward = new HashMap<Class, Map<Class, Converter>>();
    private static Map<Class, Map<Class, Converter>> reverse = new HashMap<Class, Map<Class, Converter>>();

    static {
        for (ConverterProvider provider : ServiceLoader.load(ConverterProvider.class))
            for (ConverterDescription desc : provider.getConverterDescriptions())
                add(desc.converter, desc.forwardType, desc.reverseType);
    }

    public static void add(Converter converter, Class forwardType, Class reverseType) {
        Map<Class, Converter> forwards = forward.get(forwardType);
        if (forwards == null) {
            forwards = new HashMap<Class, Converter>();
            forward.put(forwardType, forwards);
        }
        forwards.put(reverseType, converter);

        Map<Class, Converter> reverses = reverse.get(reverseType);
        if (reverses == null) {
            reverses = new HashMap<Class, Converter>();
            reverse.put(reverseType, reverses);
        }
        reverses.put(forwardType, converter);
    }

    public static <T> T convert(Object object, Class<? extends T> convertToType) throws ConversionException {
        Map<Class, Converter> forwards = forward.get(convertToType);
        if (forwards != null) {
            Converter c = forwards.get(object.getClass());
            if (c != null)
                return (T) c.convertReverse(object);
        }

        Map<Class, Converter> reverses = reverse.get(convertToType);
        if (reverses != null) {
            Converter c = reverses.get(object.getClass());
            if (c != null)
                return (T) c.convertForward(object);
        }

        return (T) object;
    }

}
